@@ -33,9 +33,11 @@
 // - use more HildonCaption in settings dialog?
 //   - checkbox buttons look fine without Caption
 //   - PickerButton could use a larger font
+// - add reset button to restore default settings
+// - set default selector in interval picker dialog
 // - check settings dialog for object/memory leaks
+// - check return code from g_timeout_add_seconds/g_signal_connect
 // - set .so version
-// - add timers to cancel async download operations
 // - support multiple instances
 
 
@@ -70,7 +72,7 @@
 
 
 
-#define DEBUG
+// #define DEBUG
 
 #ifdef DEBUG
 
@@ -84,7 +86,7 @@
 
 // check g_debug statements for syntax errors but don't emit code
 // (compiler should optimize it away)
-#define g_debug(fmt...) do { g_message(fmt); } while (0)
+#define g_debug(fmt...) do { if (0) { g_message(fmt); } } while (0)
 
 #define USER_AGENT_VERSION_STRING (some version number)
 
@@ -103,153 +105,6 @@ HD_DEFINE_PLUGIN_MODULE (CCCHHDoorIsDesktopWidget, ccchh_dooris_desktop_widget,
 
 
 
-static void
-load_settings(CCCHHDoorIsDesktopWidget *self)
-{
-	GError *error;
-
-	error = NULL;
-
-	g_debug("ccchh-dooris-widget: load_settings()");
-
-	gchar *strval = gconf_client_get_string(self->gconf_client,
-		GCONF_DIR"/update_url", &error);
-
-	if (error) {
-		g_warning("ccchh-dooris-widget: failed to get configuration value update_url: %s",
-			error->message);
-		g_error_free(error);
-		error = NULL;
-	} else if (strval) {
-		if (self->url) {
-			free(self->url);
-			self->url = NULL;
-		}
-
-		self->url = gnome_vfs_make_uri_from_input(strval);
-		g_debug("ccchh-dooris-widget: load_settings: url = %s", self->url);
-
-		g_free(strval);
-		strval = NULL;
-	}
-
-	gint intval = gconf_client_get_int(self->gconf_client,
-		GCONF_DIR"/update_interval", &error);
-
-	if (error) {
-		g_warning("ccchh-dooris-widget: failed to get configuration value update_interval: %s",
-			error->message);
-		g_error_free(error);
-		error = NULL;
-	} else if (intval) {
-		self->update = intval;
-		if (self->update < UPDATE_INTERVAL_MIN) {
-			g_warning("ccchh-dooris-widget: self->update < UPDATE_INTERVAL_MIN, maybe a bug?");
-			self->update = UPDATE_INTERVAL_MIN;
-		} else if (self->update > UPDATE_INTERVAL_MAX) {
-			g_warning("ccchh-dooris-widget: self->update > UPDATE_INTERVAL_MAX, maybe a bug?");
-			self->update = UPDATE_INTERVAL_MAX;
-		}
-		g_debug("ccchh-dooris-widget: load_settings: update = %ld", (long)self->update);
-	}
-
-	gboolean boolval = gconf_client_get_bool(self->gconf_client,
-		GCONF_DIR"/notify_enabled", &error);
-
-	if (error) {
-		g_warning("ccchh-dooris-widget: failed to get configuration value notify_enabled: %s",
-			error->message);
-		g_error_free(error);
-		error = NULL;
-	} else {
-		self->notify = boolval;
-		g_debug("ccchh-dooris-widget: load_settings: notify = %d", self->notify);
-	}
-
-
-	boolval = gconf_client_get_bool(self->gconf_client,
-		GCONF_DIR"/use_connection", &error);
-
-	if (error) {
-		g_warning("ccchh-dooris-widget: failed to get configuration value use_connection: %s",
-			error->message);
-		g_error_free(error);
-		error = NULL;
-	} else {
-		self->use_connection = boolval;
-		g_debug("ccchh-dooris-widget: load_settings: use_connection = %d", self->use_connection);
-	}
-
-	boolval = gconf_client_get_bool(self->gconf_client,
-		GCONF_DIR"/force_connection", &error);
-
-	if (error) {
-		g_warning("ccchh-dooris-widget: failed to get configuration value force_connection: %s",
-			error->message);
-		g_error_free(error);
-		error = NULL;
-	} else {
-		self->force_connection = boolval;
-		g_debug("ccchh-dooris-widget: load_settings: force_connection = %d", self->force_connection);
-	}
-}
-
-
-static void
-save_settings(CCCHHDoorIsDesktopWidget *self)
-{
-	GError *error;
-
-	error = NULL;
-
-	g_debug("ccchh-dooris-widget: save_settings()");
-
-	if (!gconf_client_set_string(self->gconf_client,
-		GCONF_DIR"/update_url", self->url, &error))
-	{
-		g_warning("ccchh-dooris-widget: failed to set configuration value update_url: %s",
-			error->message);
-		g_error_free(error);
-		error = NULL;
-	}
-
-	if (!gconf_client_set_int(self->gconf_client,
-		GCONF_DIR"/update_interval", self->update, &error))
-	{
-		g_warning("ccchh-dooris-widget: failed to set configuration value update_interval: %s",
-			error->message);
-		g_error_free(error);
-		error = NULL;
-	}
-
-	if (!gconf_client_set_bool(self->gconf_client,
-		GCONF_DIR"/notify_enabled", self->notify, &error))
-	{
-		g_warning("ccchh-dooris-widget: failed to set configuration value notify_enabled: %s",
-			error->message);
-		g_error_free(error);
-		error = NULL;
-	}
-
-	if (!gconf_client_set_bool(self->gconf_client,
-		GCONF_DIR"/use_connection", self->use_connection, &error))
-	{
-		g_warning("ccchh-dooris-widget: failed to set configuration value use_connection: %s",
-			error->message);
-		g_error_free(error);
-		error = NULL;
-	}
-
-	if (!gconf_client_set_bool(self->gconf_client,
-		GCONF_DIR"/force_connection", self->force_connection, &error))
-	{
-		g_warning("ccchh-dooris-widget: failed to set configuration value force_connection: %s",
-			error->message);
-		g_error_free(error);
-		error = NULL;
-	}
-}
-
 // stop update timer
 // returns TRUE if timer was stopped
 // returns FALSE if no timer was running
@@ -276,14 +131,14 @@ start_update_timer(CCCHHDoorIsDesktopWidget *self)
 {
 	if (!self->timeout_handler) {
 		g_debug("ccchh-dooris-widget: starting update timer");
-		if (self->update < UPDATE_INTERVAL_MIN) {
-			g_warning("ccchh-dooris-widget: self->update < UPDATE_INTERVAL_MIN, maybe a bug?");
-			self->update = UPDATE_INTERVAL_MIN;
-		} else if (self->update > UPDATE_INTERVAL_MAX) {
-			g_warning("ccchh-dooris-widget: self->update > UPDATE_INTERVAL_MAX, maybe a bug?");
-			self->update = UPDATE_INTERVAL_MAX;
+		if (self->update_interval < UPDATE_INTERVAL_MIN) {
+			g_warning("ccchh-dooris-widget: self->update_interval < UPDATE_INTERVAL_MIN, maybe a bug?");
+			self->update_interval = UPDATE_INTERVAL_MIN;
+		} else if (self->update_interval > UPDATE_INTERVAL_MAX) {
+			g_warning("ccchh-dooris-widget: self->update_interval > UPDATE_INTERVAL_MAX, maybe a bug?");
+			self->update_interval = UPDATE_INTERVAL_MAX;
 		}
-		self->timeout_handler = g_timeout_add_seconds (self->update*60, ccchh_dooris_desktop_widget_on_timeout, self);
+		self->timeout_handler = g_timeout_add_seconds (self->update_interval*60, ccchh_dooris_desktop_widget_on_timeout, self);
 		return TRUE;
 	}
 	return FALSE;
@@ -333,6 +188,153 @@ remove_connection_handler(CCCHHDoorIsDesktopWidget *self)
 
 
 
+static void
+load_settings(CCCHHDoorIsDesktopWidget *self)
+{
+	GError *error;
+
+	error = NULL;
+
+	g_debug("ccchh-dooris-widget: load_settings()");
+
+	gchar *strval = gconf_client_get_string(self->gconf_client,
+		GCONF_DIR"/update_url", &error);
+
+	if (error) {
+		g_warning("ccchh-dooris-widget: failed to get configuration value update_url: %s",
+			error->message);
+		g_error_free(error);
+		error = NULL;
+	} else if (strval) {
+		if (self->update_url) {
+			free(self->update_url);
+			self->update_url = NULL;
+		}
+
+		self->update_url = gnome_vfs_make_uri_from_input(strval);
+		g_debug("ccchh-dooris-widget: load_settings: url = %s", self->update_url);
+
+		g_free(strval);
+		strval = NULL;
+	}
+
+	gint intval = gconf_client_get_int(self->gconf_client,
+		GCONF_DIR"/update_interval", &error);
+
+	if (error) {
+		g_warning("ccchh-dooris-widget: failed to get configuration value update_interval: %s",
+			error->message);
+		g_error_free(error);
+		error = NULL;
+	} else if (intval) {
+		self->update_interval = intval;
+		if (self->update_interval < UPDATE_INTERVAL_MIN) {
+			g_warning("ccchh-dooris-widget: self->update_interval < UPDATE_INTERVAL_MIN, maybe a bug?");
+			self->update_interval = UPDATE_INTERVAL_MIN;
+		} else if (self->update_interval > UPDATE_INTERVAL_MAX) {
+			g_warning("ccchh-dooris-widget: self->update_interval > UPDATE_INTERVAL_MAX, maybe a bug?");
+			self->update_interval = UPDATE_INTERVAL_MAX;
+		}
+		g_debug("ccchh-dooris-widget: load_settings: update = %ld", (long)self->update_interval);
+	}
+
+	gboolean boolval = gconf_client_get_bool(self->gconf_client,
+		GCONF_DIR"/notify_enabled", &error);
+
+	if (error) {
+		g_warning("ccchh-dooris-widget: failed to get configuration value notify_enabled: %s",
+			error->message);
+		g_error_free(error);
+		error = NULL;
+	} else {
+		self->notify_enabled = boolval;
+		g_debug("ccchh-dooris-widget: load_settings: notify = %d", self->notify_enabled);
+	}
+
+
+	boolval = gconf_client_get_bool(self->gconf_client,
+		GCONF_DIR"/use_connection", &error);
+
+	if (error) {
+		g_warning("ccchh-dooris-widget: failed to get configuration value use_connection: %s",
+			error->message);
+		g_error_free(error);
+		error = NULL;
+	} else {
+		self->use_connection = boolval;
+		g_debug("ccchh-dooris-widget: load_settings: use_connection = %d", self->use_connection);
+	}
+
+	boolval = gconf_client_get_bool(self->gconf_client,
+		GCONF_DIR"/force_connection", &error);
+
+	if (error) {
+		g_warning("ccchh-dooris-widget: failed to get configuration value force_connection: %s",
+			error->message);
+		g_error_free(error);
+		error = NULL;
+	} else {
+		self->force_connection = boolval;
+		g_debug("ccchh-dooris-widget: load_settings: force_connection = %d", self->force_connection);
+	}
+}
+
+
+static void
+save_settings(CCCHHDoorIsDesktopWidget *self)
+{
+	GError *error;
+
+	error = NULL;
+
+	g_debug("ccchh-dooris-widget: save_settings()");
+
+	if (!gconf_client_set_string(self->gconf_client,
+		GCONF_DIR"/update_url", self->update_url, &error))
+	{
+		g_warning("ccchh-dooris-widget: failed to set configuration value update_url: %s",
+			error->message);
+		g_error_free(error);
+		error = NULL;
+	}
+
+	if (!gconf_client_set_int(self->gconf_client,
+		GCONF_DIR"/update_interval", self->update_interval, &error))
+	{
+		g_warning("ccchh-dooris-widget: failed to set configuration value update_interval: %s",
+			error->message);
+		g_error_free(error);
+		error = NULL;
+	}
+
+	if (!gconf_client_set_bool(self->gconf_client,
+		GCONF_DIR"/notify_enabled", self->notify_enabled, &error))
+	{
+		g_warning("ccchh-dooris-widget: failed to set configuration value notify_enabled: %s",
+			error->message);
+		g_error_free(error);
+		error = NULL;
+	}
+
+	if (!gconf_client_set_bool(self->gconf_client,
+		GCONF_DIR"/use_connection", self->use_connection, &error))
+	{
+		g_warning("ccchh-dooris-widget: failed to set configuration value use_connection: %s",
+			error->message);
+		g_error_free(error);
+		error = NULL;
+	}
+
+	if (!gconf_client_set_bool(self->gconf_client,
+		GCONF_DIR"/force_connection", self->force_connection, &error))
+	{
+		g_warning("ccchh-dooris-widget: failed to set configuration value force_connection: %s",
+			error->message);
+		g_error_free(error);
+		error = NULL;
+	}
+}
+
 
 static void
 draw (CCCHHDoorIsDesktopWidget *self, cairo_t *cr)
@@ -508,15 +510,31 @@ fetch_update_start (CCCHHDoorIsDesktopWidget *self)
 		return;
 	}
 
-	g_debug("ccchh-dooris-widget: begin update from URL: %s", self->url);
+	g_debug("ccchh-dooris-widget: begin update from URL: %s", self->update_url);
 
-	// TODO: set timeout to cancel async open
+	self->gvfs_timeout_handler = g_timeout_add_seconds (TIMEOUT_CONNECT, fetch_update_cancel_cb, self);
 
-	gnome_vfs_async_open(&self->gvfs_handle, self->url, GNOME_VFS_OPEN_READ,
+	gnome_vfs_async_open(&self->gvfs_handle, self->update_url, GNOME_VFS_OPEN_READ,
 		GNOME_VFS_PRIORITY_MIN, fetch_update_async_open_cb, self);
 }
 
 
+static gboolean
+fetch_update_cancel_cb (gpointer data)
+{
+	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET (data);
+  
+	g_debug("ccchh-dooris-widget: fetch_update_cancel_cb()");
+
+	g_warning("ccchh-dooris-widget: I/O timeout, cancelling operation");
+	gnome_vfs_async_cancel(self->gvfs_handle);
+
+	fetch_update_process(self, DOORIS_FNORD, 0L, 0L);
+
+	return FALSE;
+}
+
+	
 // handle async open callback, begin read
 static void
 fetch_update_async_open_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
@@ -526,6 +544,11 @@ fetch_update_async_open_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
 
 	g_debug("ccchh-dooris-widget: fetch_update_async_open_cb()");
 
+	if (self->gvfs_timeout_handler) {
+		g_source_remove (self->gvfs_timeout_handler);
+		self->gvfs_timeout_handler = 0;
+	}
+
 	if (result != GNOME_VFS_OK) {
 		g_warning("ccchh-dooris-widget: error opening URL: %s", gnome_vfs_result_to_string (result));
 		fetch_update_process(self, DOORIS_FNORD, 0L, 0L);
@@ -541,7 +564,7 @@ fetch_update_async_open_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
 	// initialize input buffer with zeroes, so strtok won't read beyond its end
 	memset(self->gvfs_input_buffer, 0, MAX_HTTP_SIZE+1);
 
-	// TODO: set timeout to cancel async read
+	self->gvfs_timeout_handler = g_timeout_add_seconds (TIMEOUT_READ, fetch_update_cancel_cb, self);
 
 	gnome_vfs_async_read (handle, self->gvfs_input_buffer, MAX_HTTP_SIZE,
 		fetch_update_async_read_cb, self);
@@ -558,6 +581,11 @@ fetch_update_async_read_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
 
 	g_debug("ccchh-dooris-widget: fetch_update_async_read_cb()");
 
+	if (self->gvfs_timeout_handler) {
+		g_source_remove (self->gvfs_timeout_handler);
+		self->gvfs_timeout_handler = 0;
+	}
+
 	if (result != GNOME_VFS_OK) {
 		g_warning("ccchh-dooris-widget: read error: %s", gnome_vfs_result_to_string (result));
 		fetch_update_process(self, DOORIS_FNORD, 0L, 0L);
@@ -566,7 +594,7 @@ fetch_update_async_read_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
 
 	g_debug("ccchh-dooris-widget: read %"GNOME_VFS_SIZE_FORMAT_STR" bytes", bytes_read);
 
-	// TODO: set timeout to cancel close
+//	self->gvfs_timeout_handler = g_timeout_add_seconds (TIMEOUT_CLOSE, fetch_update_cancel_cb, self);
 
 	// close file
 	gnome_vfs_async_close (handle, fetch_update_async_close_cb, self);
@@ -637,10 +665,15 @@ static void
 fetch_update_async_close_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
 	gpointer data)
 {
-//	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET (data);
+	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET (data);
 
 	g_debug("ccchh-dooris-widget: fetch_update_async_close_cb()");
 
+	if (self->gvfs_timeout_handler) {
+		g_source_remove (self->gvfs_timeout_handler);
+		self->gvfs_timeout_handler = 0;
+	}
+
 	if (result != GNOME_VFS_OK) {
 		g_debug("ccchh-dooris-widget: error on close: %s", gnome_vfs_result_to_string (result));
 		// TODO: what now?
@@ -705,7 +738,7 @@ fetch_update_process (CCCHHDoorIsDesktopWidget *self, dooris_state_t new_state,
 		force_redraw(self);
 
 		// flash banner if notifications are enabled
-		if (self->notify) {
+		if (self->notify_enabled) {
 			self->force_banner = TRUE;
 		}
 	}
@@ -942,7 +975,7 @@ ccchh_dooris_desktop_widget_connection_event(ConIcConnection *connection,
 
 		case CON_IC_STATUS_NETWORK_UP:
 
-			// Nothing to do here
+			// nothing to do here
 			break;
 	}
 }
@@ -964,20 +997,20 @@ ccchh_dooris_desktop_widget_on_settings_dialog_response (GtkDialog *dialog,
 		g_debug("ccchh-dooris-widget: response accept");
 
 		// status URL
-		if (self->url) {
-			free(self->url);
-			self->url = NULL;
+		if (self->update_url) {
+			free(self->update_url);
+			self->update_url = NULL;
 		}
 
 		// try to build a valid URI from user input
-		self->url = gnome_vfs_make_uri_from_input(gtk_entry_get_text(GTK_ENTRY (self->url_entry)));
-		g_debug("ccchh-dooris-widget: new URL: %s\n", self->url);
+		self->update_url = gnome_vfs_make_uri_from_input(gtk_entry_get_text(GTK_ENTRY (self->url_entry)));
+		g_debug("ccchh-dooris-widget: new URL: %s\n", self->update_url);
 
 
 		// update interval
 		const gchar *choice = hildon_button_get_value(HILDON_BUTTON (self->update_picker));
-		self->update = CLAMP(g_ascii_strtod (choice, NULL), UPDATE_INTERVAL_MIN, UPDATE_INTERVAL_MAX);
-		g_debug("ccchh-dooris-widget: new update interval: %ld\n", (long)self->update);
+		self->update_interval = CLAMP(g_ascii_strtod (choice, NULL), UPDATE_INTERVAL_MIN, UPDATE_INTERVAL_MAX);
+		g_debug("ccchh-dooris-widget: new update interval: %ld\n", (long)self->update_interval);
 
 		// if a timer was running, reset it
 		if (stop_update_timer(self)) {
@@ -985,8 +1018,8 @@ ccchh_dooris_desktop_widget_on_settings_dialog_response (GtkDialog *dialog,
 		}
 
 		// banner notifications
-		self->notify = hildon_check_button_get_active(HILDON_CHECK_BUTTON(self->notify_button));
-		if (self->notify) {
+		self->notify_enabled = hildon_check_button_get_active(HILDON_CHECK_BUTTON(self->notify_button));
+		if (self->notify_enabled) {
 			g_debug("ccchh-dooris-widget: notification enabled");
 		} else {
 			g_debug("ccchh-dooris-widget: notification disabled");
@@ -1146,16 +1179,16 @@ ccchh_dooris_desktop_widget_on_show_settings (GtkWidget *widget G_GNUC_UNUSED,
 	}
 
 	// set current values
-	gtk_entry_set_text(GTK_ENTRY(self->url_entry), self->url);
+	gtk_entry_set_text(GTK_ENTRY(self->url_entry), self->update_url);
 
 	// no default selection
 	// TODO: find the selector nearest to the current value and activate it
 	hildon_picker_button_set_active (HILDON_PICKER_BUTTON (self->update_picker), -1);
-	char *value_text = g_strdup_printf ("%d", self->update);
+	char *value_text = g_strdup_printf ("%d", self->update_interval);
 	hildon_button_set_value(HILDON_BUTTON (self->update_picker), value_text);
 	g_free(value_text);
 
-	hildon_check_button_set_active(HILDON_CHECK_BUTTON(self->notify_button), self->notify);
+	hildon_check_button_set_active(HILDON_CHECK_BUTTON(self->notify_button), self->notify_enabled);
 	hildon_check_button_set_active(HILDON_CHECK_BUTTON(self->connection_button), self->use_connection);
 	hildon_check_button_set_active(HILDON_CHECK_BUTTON(self->connforce_button), self->force_connection);
 
@@ -1215,6 +1248,11 @@ ccchh_dooris_desktop_widget_finalize (GObject *object)
   
 	remove_connection_handler(self);
 
+	if (self->gvfs_timeout_handler) {
+		g_source_remove (self->gvfs_timeout_handler);
+		self->gvfs_timeout_handler = 0;
+	}
+
 	// destroy icon surface if we still hold one
 	if (self->image_current && cairo_surface_get_reference_count(self->image_current) > 0) {
 		cairo_surface_destroy(self->image_current);
@@ -1223,9 +1261,9 @@ ccchh_dooris_desktop_widget_finalize (GObject *object)
 	// don'shut down gnome vfs library here
 	// other parts of the desktop might still use it
  
-	if (self->url) {
-		free(self->url);
-		self->url = NULL;
+	if (self->update_url) {
+		free(self->update_url);
+		self->update_url = NULL;
 	}
 
 	g_static_mutex_free(&(self->button_lock));
@@ -1297,14 +1335,12 @@ ccchh_dooris_desktop_widget_init (CCCHHDoorIsDesktopWidget *self)
 */
  
 	// default settings
-	self->notify = TRUE;
-	self->update = 23;
+	self->notify_enabled = TRUE;
+	self->update_interval = 23;
 	// TODO: this should be moved to turing.hamburg.ccc.de
-	self->url = gnome_vfs_make_uri_from_input("http://dooris.koalo.de/door.txt");
-//	self->url = g_strdup("http://dooris.koalo.de/door.txt");
-//	self->url = gnome_vfs_make_uri_from_input("http://formularfetischisten.de/~packbart/lustitsch/whale.mov");
-//	self->url = g_strdup("http://formularfetischisten.de/~packbart/temp/door.txt");
-//	self->url = gnome_vfs_make_uri_from_input("http://formularfetischisten.de/~packbart/temp/door.txt");
+	self->update_url = gnome_vfs_make_uri_from_input("http://dooris.koalo.de/door.txt");
+//	self->update_url = gnome_vfs_make_uri_from_input("http://formularfetischisten.de/~packbart/lustitsch/whale.mov");
+//	self->update_url = gnome_vfs_make_uri_from_input("http://formularfetischisten.de/~packbart/temp/door.txt");
 	self->use_connection = TRUE;
 	self->force_connection = TRUE;
 
@@ -1333,6 +1369,7 @@ ccchh_dooris_desktop_widget_init (CCCHHDoorIsDesktopWidget *self)
 
 	self->gvfs_handle = NULL;
 	self->gvfs_input_buffer = NULL;
+	self->gvfs_timeout_handler = 0;
 
 	// set widget size
 	// TODO: is this the right way?
@@ -1379,7 +1416,7 @@ ccchh_dooris_desktop_widget_init (CCCHHDoorIsDesktopWidget *self)
 		self->connected = TRUE;
 		start_update_timer(self);
 	}
-	g_debug("ccchh-dooris-widget: initial URL: %s\n", self->url);
+	g_debug("ccchh-dooris-widget: initial URL: %s\n", self->update_url);
 }
 
 
