@@ -7,6 +7,11 @@
 // gconf base path
 #define GCONF_DIR "/apps/maemo/ccchh-dooris-widget"
 
+// async I/O timeouts
+#define TIMEOUT_CONNECT 5
+#define TIMEOUT_READ	30
+#define TIMEOUT_CLOSE	5
+
 // max download size
 #define MAX_HTTP_SIZE 1024
 
@@ -71,6 +76,7 @@ typedef struct {
 	gint timeout_handler;
 	ConIcConnection *connection;
 	gint connection_handler;
+	gint gvfs_timeout_handler;
 
 	// internal state
 	cairo_surface_t *image_current;
@@ -91,9 +97,9 @@ typedef struct {
 	gulong last_report_ts;
 
 	// configuration settings
-	guint update;
-	gchar *url;
-	gboolean notify;
+	guint update_interval;
+	gchar *update_url;
+	gboolean notify_enabled;
 	gboolean use_connection;
 	gboolean force_connection;
 
@@ -129,6 +135,9 @@ ccchh_dooris_desktop_widget_connection_event(ConIcConnection *connection,
 	ConIcConnectionEvent *event, gpointer user_data);
 
 
+static gboolean
+fetch_update_cancel_cb (gpointer data);
+
 static void
 fetch_update_async_open_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
 	gpointer data);
